﻿using System;
using IndicesPerformanceTestApp.Domain;
using Telerik.Storage.Extensions;
using System.Diagnostics;

namespace IndicesPerformanceTestApp.Models
{
    public class MainPageModel
    {
        //REMARK: We will open and close database context for every one of the measured test, 
        //just to ensure that the local cash of context is empty. 
        //This is not a good practice in production code but is required for exact test results.
        private Context context;

        public MainPageModel()
        {
            //context = new Context("IndicesPerformanceTestDB", DatabaseLocation.Local);
        }

       
        
        public double CreateAndFillDB(bool bFillInIndexedTable)
        {
            //REMARK: We open and close database context just to ensure that the local cach of context is empty.
            //This is not a good practice in general.
            context = new Context("IndicesPerformanceTestDB", DatabaseLocation.Local);

            DateTime dateBegin = new System.DateTime(2010, 3, 15);
            int rndDaysRange = (int)(System.DateTime.Today - dateBegin).TotalDays;
            int age;
            DateTime bDate = DateTime.Now;

            int i = 0;
            for (i = 0; i < 30000; ++i)
            {
                age = i % 80;
                bDate.AddYears(-age);

                if (bFillInIndexedTable)
                {
                    context.Insert(new IndexedPerson()
                    {
                        Name = "Person Name " + i % 5000,//this will guarantie at least 6 records with the same name evently dustributed 
                        Age = age,
                        BornDate = bDate,
                        PersonHeight = 100 + i % 90
                    });
                }
                else 
                {
                    context.Insert(new Person()
                    {
                        Name = "Person Name " + i % 5000,//this will guarantie at least 6 records with the same name evently dustributed 
                        Age = age,
                        BornDate = bDate,
                        PersonHeight = 100 + i % 90
                    });
                }
            }

            //to measure the real data persistence performance of the engine
            Stopwatch watch = new Stopwatch();
            watch.Start();
            context.SaveChanges(); 
            watch.Stop();

            context.CloseDatabase();

            return watch.Elapsed.TotalMilliseconds;

        }

        public double RunQuery(string query, bool onIndexedTable, out long recordsCount)
        {
            context = new Context("IndicesPerformanceTestDB", DatabaseLocation.Local);

            //REMARK: we don't want to measure the initial context initialization that includes in memory schema and entity objects creation
            //so let's run some dummy query on table of interest.
            if (onIndexedTable)
            {
                context.Get<IndexedPerson>("Select * from IndexedPerson where Name = 'Person Name 1'");
            }
            else
            {
                context.Get<Person>("Select * from Person where Name = 'Person Name 1'");
            }


            Stopwatch watch = new Stopwatch();
            watch.Start();

            if (onIndexedTable)
            {
                recordsCount = context.Get<IndexedPerson>(query).Count;
            }
            else 
            {
                recordsCount = context.Get<Person>(query).Count;
            }
            
            watch.Stop();
            context.CloseDatabase();

            return watch.Elapsed.TotalMilliseconds; ;
        }


    }
}
